/// <reference types="knockout" />
import { Style, StyleObservable } from "../style/style";
import { Attribute } from "./attribute";
import { IEventListener } from "./eventListener";
import * as types from "../types/types";
export interface UiElementOptions<TChild extends UiElement = UiElement> {
    id?: string;
    elementType?: string;
    visible?: boolean | KnockoutObservable<boolean>;
    className?: string | KnockoutObservable<string>;
    attributes?: Array<Attribute>;
    eventListeners?: Array<IEventListener>;
    style?: StyleObservable;
    selectable?: boolean;
    addControlToDataDictionary?: boolean;
    children?: Array<TChild> | KnockoutObservableArray<TChild>;
}
export declare class UiElement<TChild extends UiElement<TChild> = UiElement<any>> {
    constructor(tagName: string, elementType: string, options?: UiElementOptions<TChild>);
    private display;
    private setVisibility;
    private knockoutSubscriptions;
    protected options: UiElementOptions<TChild>;
    protected build(): void;
    protected buildChildren(): void;
    render(): HTMLElement;
    refresh(): void;
    handleWebsocketMessage(websocketMessage: any): void;
    remove(): void;
    getAttribute(attributeName: types.AttributeName): string | number;
    setAttribute(name: types.AttributeName, value: string | number | KnockoutObservable<string | number>): void;
    deleteAttribute(attributeName: string): void;
    getStyle(...cssProperties: Array<types.CssProperty>): Style;
    setStyle(newStyle: Style, overwriteExisting?: boolean): void;
    deleteStyle(style: types.CssProperty | Array<types.CssProperty> | Style): void;
    addEventListener<TType extends keyof HTMLElementEventMap>(type: TType, listener: (this: HTMLElement, event: HTMLElementEventMap[TType], uiElement: UiElement) => any, options?: boolean | AddEventListenerOptions): void;
    addSubscription(observable: KnockoutObservable<any>, callback: (newValue: any) => void): void;
    getChildren(): Array<TChild>;
    setChildren(children: Array<TChild>): void;
    addChild(newChild: TChild, referenceChild?: TChild): void;
    removeChild(element: TChild): void;
    removeChildren(): void;
    protected element: HTMLElement;
    readonly tagName: string;
    readonly visible: KnockoutObservable<boolean>;
    readonly attributes: {
        [index: string]: string | number | KnockoutObservable<string | number>;
    };
}
